library ieee ;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity mf is
  generic( 
    iInDataWidth    : integer := 8;
    iOutDataWidth   : integer := 16;
    iOverSampleRate : integer := 16;
    iNumOfChips     : integer := 128
  );
  port( 
    InCode : in  std_logic ;
    InData : in  std_logic_vector (iInDataWidth-1 downto 0);
    SysClk : in  std_logic;
    SysRst : in  std_logic;
    Result : out std_logic_vector (iOutDataWidth-1 downto 0)
  );
end mf ;

architecture virtex of mf is

  type ResultArray is array (iNumOfChips-1 downto 0)
    of std_logic_vector(iOutDataWidth-1 downto 0);

  signal Zero     : std_logic_vector(iOutDataWidth-1 downto 0);
  signal lOutCode : std_logic_vector(iNumOfChips-1 downto 0);
  signal lResult  : ResultArray;

  component mf_parallel_tap
    generic (
      iInDataWidth    : integer := 8;
      iOutDataWidth   : integer := 16;
      iOverSampleRate : integer := 16
      );
    port (
      InCode  : in  std_logic;
      InData  : in  std_logic_vector (iInDataWidth-1 downto 0);
      PrevTap : in  std_logic_vector (iOutDataWidth-1 downto 0);
      SysClk  : in  std_logic;
      SysRst  : in  std_logic;
      OutCode : out std_logic;
      Result  : out std_logic_vector (iOutDataWidth-1 downto 0)
      );
  end component;

begin

  Zero <= (others => '0');

  FirstTap : mf_parallel_tap
    generic map (
      iInDataWidth    => iInDataWidth,
      iOutDataWidth   => iOutDataWidth,
      iOverSampleRate => iOverSampleRate
    )
    port map (
      InCode  => InCode,
      InData  => InData,
      PrevTap => Zero,
      SysClk  => SysClk,
      SysRst  => SysRst,
      OutCode => lOutCode(0),
      Result  => lResult(0)
    );

  InstTaps: for I in 1 to iNumOfChips-1 generate
    Taps : mf_parallel_tap
      generic map (
        iInDataWidth    => iInDataWidth,
        iOutDataWidth   => iOutDataWidth,
        iOverSampleRate => iOverSampleRate
      )
    port map (
        InCode  => lOutCode(I-1),
        InData  => InData,
        PrevTap => lResult(I-1),
        SysClk  => SysClk,
        SysRst  => SysRst,
        OutCode => lOutCode(I),
        Result  => lResult(I)
      );
  end generate InstTaps;

  GenerateResult : Result <= lResult(iNumOfChips-1);

end virtex;
